/*
 * 	 amoeba.java
 * 
 * 
 *   famCNV: Copy Number Variant association for quantitative traits in families
 *   Copyright (C) 2010 Dr. Mario Falchi                                   

 *
 *   famCNV is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   famCNV is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with famCNV.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   For any bugs or problems found, please contact us at
 *   m.falchi@imperial.ac.uk  
 *	 hariklia.eleftherohorinou06@imperial.ac.uk
 *   visconti@di.unito.it
 */

class Amoeba 
{	
	private double[][] directions;
	private double[][] simplex;
	private double[] point;
	
	private double[] y;
	private double[] psum;
	private double[] ptry;
	private double fmin;

	private NormalSet caller;

	Amoeba(NormalSet caller, int dim) 
	{
		this.caller = caller;
		
		point = new double[dim];
		directions = new double[dim][dim];
		
		simplex = new double[dim+1][dim];
		y = new double[dim+1];
		psum = new double[dim];
		ptry = new double[dim];
		
		fmin = Double.MAX_VALUE;
	}

	public double[] point()
	{
		return point;
	}
	
	public double point(int index)
	{
		return point[index];
	}
	
	public double fmin()
	{
		return fmin;
	}
	
	public void setPoint(int index, double value)
	{
		point[index] = value;
	}

	public void setPoint(double[] value)
	{
		point = value;
	}

	public void reset(int dim, double scale) 
	{
		
		for (int i = 0; i < dim; i++) 
			for (int j = 0; j < dim; j++) 
				if (i == j)
					directions[i][j] = 1.0 * scale;
				else
					directions[i][j] = 0.0;
	
		fmin = Double.MAX_VALUE;
	}


	public double minimize(double ftol) throws Exception 
	{
		if (point.length == 0)
			return fmin = caller.callEvaluate(point);
		
		int nvertex = point.length + 1;
		
		for (int i = 0; i < point.length; i++) 
		{
			for (int j = 0; j < simplex[i].length; j++)
				simplex[i][j] = point[j] + directions[i][j];
				
			y[i] = caller.callEvaluate(simplex[i]);
			
			if (y[i] < fmin)
				fmin = y[i];
		}
		
		for (int i =0; i<point.length; i++)
			simplex[nvertex-1][i] = point[i];
			
		y[nvertex-1] = caller.callEvaluate(simplex[nvertex-1]);		
		if (y[nvertex - 1] < fmin)
			fmin = y[nvertex - 1];
		
		long cycleCount = (long) nvertex;
		
		psum = Utilities.set(psum.length, 0.0);
		for (int m = 0; m < nvertex; m++) 
			for (int j = 0; j < psum.length; j++)
				psum[j] += simplex[m][j];
			
		while (true) 
		{
			
			int inhi, ilo, ihi;
			
			if (y[0] > y[1])
			{
				ilo = inhi = 1;
				ihi = 0;
			}
			else
			{
				ilo = inhi = 0;
				ihi = 1;
			}

			for (int i = 2; i < nvertex; i++) 
			{
				if (y[i] <= y[ilo])
					ilo = i;
				else if (y[i] > y[ihi]) 
				{
					inhi = ihi;
					ihi = i;
				} 
				else if (y[i] > y[inhi])
					inhi = i;
			}
					
			double rtol = 2.0 * Math.abs(y[ihi] - y[ilo]) / (Math.abs(y[ihi]) + Math.abs(y[ilo]) + Utilities.ZEPS);
			
			if (rtol < ftol) 
			{
				for (int i=0; i<point.length; i++)
					point[i] = simplex[ilo][i];
				
				return fmin = y[ilo];
			}
			
			if (cycleCount > Utilities.CYCLEMAX)
				throw new Exception("Amoeba.Minimize - Couldn't converge in " + Utilities.CYCLEMAX + " cycles");
			
			cycleCount += 2L;
			double ytry = Amoeba(ihi, -1.0);
				
			if (ytry <= y[ilo])
				Amoeba(ihi, 2.0);
			else if (ytry >= y[inhi]) 
			{
				double ysave = y[ihi];
				ytry = Amoeba(ihi, 0.5);
				if (ytry >= ysave) 
				{
					for (int i = 0; i < nvertex; i++) 
					{
						if (i != ilo) 
						{
							for (int s = 0; s < simplex[i].length; s++)
								simplex[i][s] += simplex[ilo][s];
							for (int s = 0; s < simplex[i].length; s++)
								simplex[i][s] *= 0.5;
							
							y[i] = caller.callEvaluate(simplex[i]);
						}
					}
					cycleCount += (long) point.length;
					
					psum = Utilities.set(psum.length, 0.0);
					for (int m = 0; m < nvertex; m++)
						for (int s = 0; s < simplex[m].length; s++)
							psum[s] += simplex[m][s];		
					
				}
			}
			else
				cycleCount--;
		
		}
	}


	private double Amoeba(int ihi, double factor) throws Exception
	{
		double fac = (1.0 - factor) / point.length;
		
		for (int s = 0; s < psum.length; s++)
			ptry[s] = fac * psum[s];
		
		double value = factor-fac;
		for (int s = 0; s < simplex[ihi].length; s++)
			ptry[s] += value * simplex[ihi][s];
		
		try 
		{
			double ytry = caller.callEvaluate(ptry);
			
			if (ytry < y[ihi]) 
			{
				y[ihi] = ytry;
				for (int s = 0; s < psum.length; s++) 
					psum[s] -= simplex[ihi][s];
				for (int s=0; s<psum.length; s++)
				{
					simplex[ihi][s] = ptry[s];
					psum[s] +=  simplex[ihi][s];
				}
			}
					
			return ytry;
		}
		catch (Exception e)
		{
			throw e;
		}
		
	}
}
