/*
 *   famCNV: Copy Number Variant association for quantitative traits in families
 *   Copyright (C) 2010 Dr. Mario Falchi                                   

 *
 *   famCNV is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   famCNV is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with famCNV.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   For any bugs or problems found, please contact us at
 *   m.falchi@imperial.ac.uk  
 *	 hariklia.eleftherohorinou06@imperial.ac.uk
 *   visconti@di.unito.it
 */



import java.util.*;
import java.io.*;
import java.lang.Runtime;


public class Main 
{

	private static String info()
	{
		String s = "famCNV: Copy Number Variant association for quantitative traits in families\n";
		s += "Copyright (C) 2010-2014 Dr. Mario Falchi <m.falchi@imperial.ac.uk>\n";
		
		return  s;
	}
	
	private static String help()
	{
		String s = "";
		
		//s += "famCNV is distributed in the hope that it will be useful,\n";
		//s += "but WITHOUT ANY WARRANTY. See the GNU GPL for more details.\n\n";
		
		s += "Datails can be found at:\n";
		s += "\tEleftherohorinou et al., Bioinformatics 27.13:1873-1875 (2011)\n";
		
		s += "\nPlease report comments and bugs to:\n";
		s += "\t- m.falchi@imperial.ac.uk\n\t- hariklia.eleftherohorinou06@imperial.ac.uk\n\t- a.visconti@imperial.ac.uk\n";
		
		s += "\n" + Constants.optionList() + "\n";
		
		return s;
	}

	public static void main(String[] args) 
	{
		try 
		{
			//read options
			Constants.parse(args);
		
			if (Constants.verbose)
				System.out.println(info());	
			
			if (Constants.help)
			{
				System.out.println(help());
				System.exit(0);
			}
			
			if (Constants.verbose)
			{
				System.out.println("\t---------------------");
				System.out.println("\t  Option summary");
				System.out.println("\t---------------------\n");
				System.out.println(Constants.printConstants());
			}
			
			//loading data
			Drop drop = new Drop();
			int numTraits = drop.readTraits(); 
			int people = drop.loadFamilies(Constants.ped, Constants.pheno, numTraits);	
			
			if (Constants.cov != null) 
				drop.readCovariates();
			int numCov = (drop.numCovar() != Utilities.INVALID_I) ? drop.numCovar() : 0;
			
			int numSNPinfo = drop.readSNPList();
			drop.readSNPs(numSNPinfo);	
			
			int numIncludedSNP = numSNPinfo;
			if (Constants.include != null)
				numIncludedSNP = drop.readIncludedSNPs();
			
			if (Constants.verbose)
			{
				System.out.println("\t---------------------");
				System.out.println("\t  Parameter summary");
				System.out.println("\t---------------------\n");
			 
				System.out.println("\t" + people + "\tindividuals\t  read from [ " + Constants.ped + " ]");
				System.out.println("\t" + drop.numFamilies() + "\tfamilies\t  read from [ " + Constants.ped + " ]");
				System.out.println("\t" + numCov + "\tcovariates\t  read from [ " + Constants.cov + " ]");

				System.out.println("\t" + numTraits + "\ttraits\t\t  read from [ " + Constants.trt + " ]");
				System.out.println("\t" + numSNPinfo + "\tintensity signals read from [ " + Constants.mrk + " ]");
			
				System.out.println();
				System.out.println("\t" + Math.min(Runtime.getRuntime().availableProcessors(), Constants.threads) + "\tthreads running");
				System.out.println("\t" + numIncludedSNP + "\tmarkers included in the analysis");
					
				if (Constants.permutations != -1)
					System.out.println("\t" + Constants.permutations + "\tpermutations will be performed");
			}
			
			if (Constants.inverse)
			{
				try
				{
					drop.inverseNormalTransform();
					if (Constants.verbose)
						System.out.println("Data normalised.");
				}
				catch(Exception e)
				{
					//System.out.println(e.getMessage());
					System.out.println("HINT: try --inverse false");
					System.out.println();
					return;
				}
			}

			drop.familyPreprocess();
			
			if (Constants.verbose)
			{
				System.out.println("\n\t---------------------\n");
				System.out.println("\nAnalysis started. Each dot corresponds to 10 tests.\n") ;
			}
			
			//doing analysis
			Vector<Result> res = drop.analyze();
		
			
			
			if (Constants.verbose)
			{	
				System.out.println();
				System.out.println("\nAnalysis ended. " + drop.tests() + " tests performed.\n") ;
			}
			
			
			//print results
			//FIXME?
			if (Constants.output != null)
			{
				System.out.println("Results in " + Constants.output + "\n");
				System.setOut(new PrintStream(new FileOutputStream(new File(Constants.output))));
			}
			
			if (Constants.header)
				System.out.println(Result.header(Constants.permutations != -1, Constants.relc != Utilities.INVALID_D, Constants.variance));
			
			for (Enumeration<Result> elements = res.elements(); elements.hasMoreElements();)
			{
				try 
				{	
					System.out.println(elements.nextElement().to_string());
				}
				catch (Exception e) 
				{
					System.out.println(e.getMessage());
				}
			}
		} 
		catch (Exception e) 
		{
			//e.printStackTrace();
			
			String msg = e.getMessage();
			if (msg.equals("-1"))
				msg = "Array Index Out Of Bounds Exception";
				
			System.out.println(msg);
			System.out.println();
		}
	}

}
