/*
 * 	 Utilities.java
 * 
 * 
 *   famCNV: Copy Number Variant association for quantitative traits in families
 *
 *   famCNV is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.

 *   famCNV is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with famCNV.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   For any bugs or problems found, please contact us at
 *   m.falchi@imperial.ac.uk  
 * 	 visconti@di.unito.it
 */

import java.util.*;
import java.text.*;

public class Utilities 
{
	public static final double INVALID_D = Double.MIN_VALUE;	
	public static final int INVALID_I = Integer.MIN_VALUE;	
	public static final long CYCLEMAX  = 50000L;			//max iterations
	public static final double TINY    = 1.0e-30;    		// A small number
	public static final double ZEPS    = 3.0e-10;    		// Precision around zero
	public static final double FPMIN   = 1.0e-30;    		// Number near the smallest representable number
	public static final double TOL     = 1.0e-6;     		// Zero SVD values below this
	
	public static final double PRECISION = 1.0E-8;
		
	public static final int VC_FULL = 2; 			//VC parameter
	public static final int VC_NULL = 2; 			//VC parameter
	
	public static final String[] VALID_TWIN = new String[] { "0", "DZ", "MZ" }; //valid twin codes
	public static final Integer[] VALID_SEX = new Integer[] {new Integer(1), new Integer(2)};						//valid sex codes
	
	
	public static boolean isValid(Object value, Object[] values)
	{
		return Arrays.asList(values).contains(value);
	}
	
	public static double[] shuffle(double[] ar)
	  {
	    Random rnd = new Random();
	    for (int i = ar.length - 1; i > 0; i--)
	    {
	      int index = rnd.nextInt(i + 1);
	      // Simple swap
	      double a = ar[index];
	      ar[index] = ar[i];
	      ar[i] = a;
	    }
		
		return ar;
	  }
	

	public static double[][] setTriangMatrix(double[][] matrix, double value, int key1, int key2) 
	{
		int dim = matrix.length -1;
		if (key1 > key2)
			matrix[key2][dim - key1] = value;
		else
			matrix[key1][dim - key2] = value;
			
		return matrix;
	}

	public static double retrieveDTriangMatrix(double[][] matrix, int key1, int key2) 
	{
		int dim = matrix.length -1;
		if (key1 > key2)
			return matrix[key2][dim - key1];
		else
			return matrix[key1][dim - key2];
	}

	public static double[][] set(int r, int c, double v) 
	{
		double[][] m = new double[r][c];
		for(int i=0; i<r; i++)
			for(int j=0; j<c; j++)
				m[i][j] = v;
		return m;
	}

	public static double innerProduct(double[] v1, double[] v2) throws Exception
	{
		if (v1.length != v2.length)
			throw new Exception("Internal error. Impossible to evaluate inner product.");
		
		double innerProduct = 0.0;
		for (int i = 0; i < v1.length; i++)
			innerProduct += v1[i] * v2[i];
		
		return innerProduct;
	}
	
	
	public static double[] set(int n, double v)
	{
		double[] r = new double[n];
		for (int i=0; i<n; i++)
			r[i] = v;
		return r;
	}
	
	public static int[] set(int n, int v)
	{
		int[] r = new int[n];
		for (int i=0; i<n; i++)
			r[i] = v;
		return r;
	}
	
	public static String printArray(double[] array, DecimalFormat df)
	{
		String s = "(";
		for (int i=0; i<array.length; i++)
			s += " " + df.format(array[i]);
		
		s += " )";
		
		return s;
	}
	
	
	public static double mean(Vector<Double> v)
	{
		double s = 0.0;
		for(int i=0; i<v.size(); i++)
			s +=  v.get(i).doubleValue();
		
		return s/v.size();
	}
	
	
	public static double variance(Vector<Double> v, boolean correct)
	{
		double mean = mean(v);
		double s = 0.0;
		for(int i=0; i<v.size(); i++)
		{
			double a = v.get(i).doubleValue();
			s += (mean-a)*(mean-a);
		}
		
		if (correct)
			return  s/(v.size()-1);
	
		return s/v.size();
	}
	
	public static double[] vector2double(Vector<Double> v)
	{
		double[] d = new double[v.size()];
		for(int i=1; i<v.size(); i++)
			d[i] = v.get(i);
		return d;
	}
	


/****************************************************
   C Equivalent of Wichura's PPND16, Algorithm AS241
   Applied Statistics Vol 37 1988 pp 477 - 484
*****************************************************/
	public static double ninv(double p) throws Exception
	{
		
	   double SPLIT1 = 0.425;
	   double SPLIT2 = 5.0;
	   double CONST1 = 0.180625;
	   double CONST2 = 1.6;
	   double HUGE_VAL = Double.MAX_VALUE;
	
	   double a[] = new double[] { 3.3871328727963666080E0, 1.3314166789178437745E2, 1.9715909503065514427E3,1.3731693765509461125E4, 4.5921953931549871457E4, 6.7265770927008700853E4, 3.3430575583588128105E4, 2.5090809287301226727E3 };
	
	   double b[] = new double[]{4.2313330701600911252E1, 6.8718700749205790830E2,  5.3941960214247511077E3,  2.1213794301586595867E4, 3.9307895800092710610E4, 2.8729085735721942674E4, 5.2264952788528545610E3 };
	
	   double c[] = new double[] { 1.42343711074968357734E0, 4.63033784615654529590E0, 5.76949722146069140550E0, 3.64784832476320460504E0, 1.27045825245236838258E0, 2.41780725177450611770E-1, 2.27238449892691845833E-2, 7.74545014278341407640E-4 };
	
	   double d[] = new double[]{ 2.05319162663775882187E0, 1.67638483018380384940E0, 6.89767334985100004550E-1, 1.48103976427480074590E-1, 1.51986665636164571966E-2, 5.47593808499534494600E-4, 1.05075007164441684324E-9 } ;
	
	  double e[] = new double[]{ 6.65790464350110377720E0, 5.46378491116411436990E0, 1.78482653991729133580E0, 2.96560571828504891230E-1, 2.65321895265761230930E-2, 1.24266094738807843860E-3, 2.71155556874348757815E-5, 2.01033439929228813265E-7 } ;
	
	   double f[] = new double[] { 5.99832206555887937690E-1, 1.36929880922735805310E-1, 1.48753612908506148525E-2, 7.86869131145613259100E-4, 1.84631831751005468180E-5,  1.42151175831644588870E-7, 2.04426310338993978564E-15 } ;
	
	   double q = p - 0.5;
	   double r;
	   double x = 0.0;
	
	   if ( Math.abs( q ) < SPLIT1 )
	   {
	      r = CONST1 - q * q ;
	      return q * ((((((( a[7] * r + a[6] ) * r + a[5] ) * r + a[4] ) * r + a[3] ) * r + a[2] ) * r + a[1] ) * r + a[0] ) /  ((((((( b[6] * r + b[5] ) * r + b[4] ) * r + b[3] ) * r  + b[2] ) * r + b[1] ) * r + b[0] ) * r + 1.0 ) ;
	   }
	   else
	   {
			if ( q < 0 )
				r = p ;
			else
				r = 1.0 - p ;
	
			if ( r < 1e-200)
				throw new Exception("ERROR: p-value " + r + " outside range in Normal Inversee Transformation");
	
			if ( r > 0.0 )
			{
				r = Math.sqrt( -Math.log( r ) ) ;
				if ( r <= SPLIT2 )
				{
					r -= CONST2 ;
					x = ((((((( c[7] * r + c[6] ) * r + c[5] ) * r + c[4] ) * r + c[3] ) * r + c[2] ) * r + c[1] ) * r + c[0] ) /  ((((((( d[6] * r + d[5] ) * r + d[4] ) * r + d[3] ) * r + d[2] ) * r + d[1] ) * r + d[0] ) * r + 1.0 ) ;
				}
				else
				{
					r -= SPLIT2 ;
					x =  ((((((( e[7] * r + e[6] ) * r + e[5] ) * r + e[4] ) * r + e[3] ) * r + e[2] ) * r + e[1] ) * r + e[0] ) /  ((((((( f[6] * r + f[5] ) * r + f[4] ) * r + f[3] ) * r + f[2] ) * r + f[1] ) * r + f[0] ) * r + 1.0 ) ;
				}
			 }
			else
				x = HUGE_VAL;
			
			if ( q < 0 )
				x = -x ;

			return x ;
	   }
	}

	
}
